@@ -1,12 +1,14 @@
+// [[Rcpp::depends(meteoland)]]
 #include <Rcpp.h>
 #include <numeric>
 #include <math.h>
-#include "swb.h"
+#include "spwb.h"
 #include "forestutils.h"
 #include "fuelstructure.h"
 #include "fuelmoisture.h"
 #include "fireutils.h"
 #include "windextinction.h"
+#include <meteoland.h>
 using namespace Rcpp;
 
 const double V = 1116.0; //Latent heat of vaporisation of water in Btu/lb
@@ -564,273 +566,275 @@ List rothermel(String modeltype, NumericVector wSI, NumericVector sSI, double de
  *  surfaceToVolumeRatios: A vector of surface-to-volume ratio (m2/m3) for five fuel classes
  *  heatContent: A vector of heat content (kJ/kg) for five fuel classes
  *  deadFuelMoistureExtinction: a value of dead fuel moisture of extinction (percent)
- *  verbose: A flag to print intermediate output
+ *  control: A list with default parameter values
  */
-List fb(List x, List soil, double latitude, double slope, double aspect, DataFrame meteo, DataFrame SpParams, 
-        DataFrame FuelModelParams, String liveFMCmode = "swb", bool useModelForLive = false, bool verbose=true) {
-  if(verbose) Rcout<<"Initializing";
-  
-  //Prepare swb input
-  DataFrame swbInput = forest2swbInput(x, SpParams, soil["dVec"]);
-  NumericVector pBole = swbInput["pBole"];
-  NumericVector H = swbInput["H"];
-  NumericVector Sgdd = swbInput["Sgdd"];
-  IntegerVector SP = swbInput["SP"];
-  NumericVector LAI = swbInput["LAI"];
-  
-  NumericVector cohLoading = cohortFuel(x, SpParams);
-
-  //Initialize other cohort-based variables
-  int numCohorts = LAI.size();
-  NumericVector LAIphe(numCohorts), cohFMC(numCohorts), cohLoadingPhe(numCohorts);
-  
-  //Meteorological variables
-  IntegerVector DOY = meteo["DOY"]; // day of the year
-  NumericVector Precipitation = meteo["Precipitation"]; // in mm of water (L/m2)
-  NumericVector MeanTemperature = meteo["MeanTemperature"]; // in degrees Celsius
-  NumericVector MinTemperature = meteo["MinTemperature"]; // in degrees Celsius
-  NumericVector MaxTemperature = meteo["MaxTemperature"]; // in degrees Celsius
-  NumericVector RHmean = meteo["MeanRelativeHumidity"]; // in percentage
-  NumericVector RHmin = meteo["MinRelativeHumidity"]; // in percentage
-  NumericVector RHmax = meteo["MaxRelativeHumidity"]; // in percentage
-  NumericVector WS = meteo["WindSpeed"]; // in m/s
-  NumericVector WD = meteo["WindDirection"]; // in radians from north
-  NumericVector Rn = meteo["Radiation"];  //in MJ/m2
-  NumericVector PET = meteo["PET"]; //in mm of water
-
-  NumericVector GDD = gdd(DOY, MeanTemperature, 5.0);
-  NumericVector ER = er(DOY);
-
-  int numDays = Precipitation.size();
-  
-  //Output FS variables
-  NumericVector fuelbedHeight(numDays), fine1hLoading(numDays), 
-                coarse10hLoading(numDays), coarse100hLoading(numDays), 
-                herbaceousLoading(numDays), woodyLoading(numDays),
-                canopyBaseHeight(numDays), canopyTopHeight(numDays),
-                canopyLength(numDays), canopyBulkDensity(numDays),
-                canopyLAI(numDays);
-
-  //Output FM variables
-  NumericVector fuelWind(numDays), fine1hFMC(numDays), coarse10hFMC(numDays), 
-                coarse100hFMC(numDays), canopyFMC(numDays), fuelbedWoodyFMC(numDays),
-                fuelbedHerbaceousFMC(numDays);
-
-  //Output FB variables
-  IntegerVector burningType(numDays);
-  NumericVector firelineIntensity(numDays), spreadRate(numDays), surfaceSpreadRate(numDays), 
-                midflameWind(numDays), virtualWindSpeed(numDays), spreadDirection(numDays), I0(numDays);
-  
-  //Initialize fuel moisture
-  NumericVector fMoisture(5);
-  fMoisture[0] = x["dead1hourMoisture"];
-  fMoisture[1] = x["dead10hourMoisture"];
-  fMoisture[2] = x["dead100hourMoisture"];
-  
-  NumericVector fcafternoonprev; //Stores fuel conditions of afternoon of previous day
-  if(verbose) Rcout<<" - Daily loop ";
-  for(int i=0;i<numDays;i++) {
-    //SOIL WATER BALANCE
-    if(verbose) Rcout<<".";
-    List s = swbDay1(swbInput, soil, GDD[i], PET[i], Precipitation[i], ER[i], 0.0); //No Runon in simulations for a single cell    
-    double Lground = s["Lground"]; //Percent light at the ground level
-
-    //FUEL STRUCTURE
-    NumericVector Phe = leafDevelopmentStatus(Sgdd, GDD[i]);
-    for(int c=0;c<numCohorts;c++) {
-      if(Sgdd[c]==0.0) Phe[c]=1.0;
-      LAIphe[c] = LAI[c]*Phe[c]; //LAI modified by phenology 
-      cohLoadingPhe[c] = cohLoading[c]*Phe[c]; //LAI modified by phenology 
-    }
-    //Loadings and FMC by cohort
-    cohFMC = cohortFuelMoistureContent(s, swbInput, SpParams);
-    
-    List fs = fuelStructure(x, SpParams, FuelModelParams, GDD[i], 30, 5000, 0.01, useModelForLive);
-    fuelbedHeight[i] = fs["fuelbedHeight [cm]"];
-    NumericVector fBiomass = fs["fuelbedLoading [Mg/ha]"];
-    NumericVector surfaceToVolumeRatios = fs["fuelbedSAV [m2/m3]"];
-    fine1hLoading[i] = fBiomass[0];
-    coarse10hLoading[i] = fBiomass[1];
-    coarse100hLoading[i] = fBiomass[2];
-    herbaceousLoading[i] = fBiomass[3];
-    woodyLoading[i] = fBiomass[4];
-    canopyBaseHeight[i] = fs["canopyBaseHeight [cm]"];
-    canopyTopHeight[i] = fs["canopyTopHeight [cm]"];
-    canopyLength[i] = fs["canopyLength [cm]"];
-    canopyBulkDensity[i] = fs["canopyBulkDensity [kg/m3]"];
-    canopyLAI[i] = fs["canopyLAI [dimensionless]"];
-
-    //WIND EXTINCTION
-    //Check NA
-    if(NumericVector::is_na(WS[i])) WS[i] = 2.0; //Default value
-    if(NumericVector::is_na(WD[i])) WD[i] = 180.0; //Wind from south
-    
-    //Calculates midflameWind
-    if(NumericVector::is_na(canopyBaseHeight[i])) { //NO canopy over fire
-       midflameWind[i] = unshelteredMidflameWindSpeed(WS[i], fuelbedHeight[i]/100.0); 
-       fuelWind[i] = windSpeedAtCanopyHeight(WS[i], fuelbedHeight[i]/100.0);
-    } else { //Wind extinction due to crowns
-       midflameWind[i] = windSpeedMassmanExtinction(canopyBaseHeight[i]/100.0,WS[i], canopyLAI[i], canopyTopHeight[i]/100.0);
-       fuelWind[i] = midflameWind[i];
-    }    
-
-    //FUEL MOISTURE
-    double netPrec = s["NetPrec"];
-    double rainDuration = std::min(24.0,Precipitation[i]/6.35);
-    
-    //Number of hours of insolation
-    double delta = 0.4093*sin((2.0*PI*((double)DOY[i])/365.0) - 1.405);
-    NumericVector srs = sunRiseSet(latitude*(PI/180.0), aspect*(PI/180.0), slope*(PI/180.0), delta);
-    double n = std::max(0.0, (srs[1] - srs[0])*12.0/PI); 
-    if(n<=0.0) n = 0.0;
-    
-    //Dead fuel moisture
-    double radSec = Rn[i]*(Lground/100.0)*(1000000.0/(n*3600.0)); //Transform to MJ/m2.day to W/m2 assuming n hours of equal sunlight
-    if(n<=0.0) radSec = 0.0;
-
-    NumericVector fcmean = fuelConditions(MeanTemperature[i],RHmean[i], radSec, fuelWind[i]);
-    NumericVector fcmorning = fuelConditions(MinTemperature[i],RHmax[i], radSec, fuelWind[i]);
-    NumericVector fcafternoon = fuelConditions(MaxTemperature[i],RHmin[i], radSec, fuelWind[i]);
-    if(i==0) fcafternoonprev = fcafternoon; 
-    fine1hFMC[i] = fine1hday(fMoisture[0], fcmean[0], fcmean[1], fuelWind[i], netPrec);
-    coarse10hFMC[i] = coarse10hday(fMoisture[1], 
-                                   (fcafternoonprev[0] +fcmorning[0])/2.0, (fcafternoonprev[1] + fcmorning[1])/2.0,
-                                   (fcmorning[0]+fcafternoon[0])/2.0, (fcmorning[1]+fcafternoon[1])/2.0, rainDuration);
-    coarse100hFMC[i] = coarse100hday(fMoisture[2], 
-                                     fcmorning[0], fcmorning[1],
-                                     fcafternoon[0], fcafternoon[1], n, rainDuration);
-    //Store for next day (and for fire behaviour)
-    fMoisture[0] = fine1hFMC[i]; 
-    fMoisture[1] =  coarse10hFMC[i]; 
-    fMoisture[2] = coarse100hFMC[i]; 
-    fcafternoonprev = fcafternoon;
-      
-    //Live fuel moisture
-    NumericVector psi = s["psiVec"];
-    if(liveFMCmode == "constant") {
-      fMoisture[3] = 200.0; 
-      fMoisture[4] = 100.0; 
-      canopyFMC[i] = 100.0;
-    } else if(liveFMCmode =="ffmc") {
-      fMoisture[3] = fine1hFMC[i]; 
-      fMoisture[4] = fine1hFMC[i]; 
-      canopyFMC[i] = fine1hFMC[i];
-    } else if(liveFMCmode =="swb") {
-      //Level of physiological activity is defined in terms of growth degree days
-      double act = leafDevelopmentStatus(200.0, GDD[i]);
-      //In winter, herbaceous FMC is modelled as fine (1h) dead fuels
-      //When physiologically active, FMC is modelled as function of water content in the topsoil
-      fMoisture[3] = act*std::max(30.0, 200.0+20*(psi[0]/1000.0))+(1.0-act)*fine1hFMC[i]; 
-      
-      fMoisture[4] = fuelbedLiveFuelMoisture(fuelbedHeight[i], cohFMC, cohLoadingPhe, H, pBole); 
-      canopyFMC[i] = canopyLiveFuelMoisture(canopyBaseHeight[i], canopyTopHeight[i], cohFMC, cohLoadingPhe, H, pBole); 
-    } else {
-      stop("Wrong 'liveFMCmode'");
-    }
-    
-    fuelbedHerbaceousFMC[i] = fMoisture[3];
-    fuelbedWoodyFMC[i] = fMoisture[4];
-
-
-   CharacterVector FMcode = x["FuelModelCode"];
-   CharacterVector models = FuelModelParams.attr("row.names");
-   NumericVector heatContent(5);
-   double deadFuelMoistureExtinction = 30.0; //Default value
-   for(int j=0;j<models.length();j++) {
-     if(models[j]==FMcode[0]) {
-       heatContent[0] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_1h"])[j];
-       heatContent[1] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_10h"])[j];
-       heatContent[2] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_100h"])[j];
-       heatContent[3] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_Live_Herb"])[j];
-       heatContent[4] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_Live_Woody"])[j];
-       deadFuelMoistureExtinction = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Mx_dead"])[j];
-     }
-   }
-   
-    //FIRE BEHAVIOUR
-    //1. Calculate surface fire characteristics
-    //1.2 Call to Rothermel model to obtain surface fire spread rate and intensity
-    List rtm = rothermel("D", fBiomass, surfaceToVolumeRatios, fuelbedHeight[i],
-                        deadFuelMoistureExtinction, heatContent, fMoisture, 
-                        midflameWind[i], WD[i], 
-                        slope, aspect);
-    surfaceSpreadRate[i] = rtm["ROS [m/min]"];
-    spreadRate[i] = surfaceSpreadRate[i];
-    firelineIntensity[i] = rtm["Fireline intensity [kW/m]"];
-    virtualWindSpeed[i] = rtm["Virtual wind speed [m/s]"];
-    NumericVector slopeWindVec = rtm["Slope-wind vector"];
-    
-    //1.3 Determine whether crowning will occur  (Van Wagner 1989)
-    if(!NumericVector::is_na(canopyBaseHeight[i])) {
-      I0[i] = criticalFirelineIntensity(canopyBaseHeight[i]/100.0, canopyFMC[i]); //critical fireline intensity (MW/m)
-      //Is crown fire (active or passive)?
-      if(firelineIntensity[i] >= I0[i]) { 
-        double RAC = 3.0/canopyBulkDensity[i]; // Threshold for active crown spread rate (m/min)
-        double Ei = 1.0;
-        NumericVector fBiomass10 = NumericVector::create(0.38, 0.092, 0.230,0.0, 0.092); //lb/square feet
-        fBiomass10 = fBiomass10*(1.0/(0.3048*0.3048))*0.45359237*(10.0); // transform to tons/hectare
-        List rtm10 = rothermel("D", fBiomass10, surfaceToVolumeRatios, 30.48,
-                        25, heatContent, fMoisture, WS[i]*0.4, WD[i], 
-                        slope, aspect);
-        double R10 = rtm10["ROS [m/min]"]; //Spread rate with fuel model 10 (Rothermel 1991)
-        double R0 = (I0[i]/firelineIntensity[i])*surfaceSpreadRate[i]; //critical surface spread rate (m/s) associated with critical fireline intensity
-        double ac = -log(0.1)/(0.9*(RAC-R0)); //Scaling coefficient
-        double cFractionBurn = 1.0 - exp(-ac*(surfaceSpreadRate[i]-R0)); //Fraction of canopy burn with crowning
-        double Rcmax = (3.34)*R10*Ei; // Maximum crown spread rate (m/s)
-        double Rcactual = surfaceSpreadRate[i] + cFractionBurn*(Rcmax - surfaceSpreadRate[i]); //Actual crown spread rate (m/s)
-        if(Rcactual >= RAC) {
-          burningType[i] = 4;
-          spreadRate[i] = Rcactual;
-        } else {
-          burningType[i] = 3;
-        }
-        //Modify fireline intensity for crown fire
-        firelineIntensity[i] = 300.0 * ((firelineIntensity[i]/(300.0*surfaceSpreadRate[i])) + cFractionBurn*canopyBulkDensity[i]*(canopyTopHeight[i]-canopyBaseHeight[i])*0.01)*spreadRate[i];        
-      } else {
-        burningType[i] = 2; // Surface fire
-      }
-    } else {
-      burningType[i] = 0; // Shrubland (open) fire
-    }
-
-    spreadDirection[i] = slopeWindVec[1]*(180.0/3.141592);  
-    if(spreadDirection[i]<0) spreadDirection[i] +=360.0; 
-  }
-  
-  if(verbose) Rcout<<"Building output ...";  
-  Rcpp::DataFrame FS = DataFrame::create(_["fuelbedHeight"] = fuelbedHeight,
-                      _["fine1hLoading"] = fine1hLoading,
-                      _["coarse10hLoading"] = coarse10hLoading,
-                      _["coarse100hLoading"] = coarse100hLoading,
-                      _["herbaceousLoading"] = herbaceousLoading,
-                      _["woodyLoading"] = woodyLoading,
-                      _["canopyBaseHeight"] = canopyBaseHeight,
-                      _["canopyTopHeight"] = canopyTopHeight,
-                      _["canopyLength"] = canopyLength,
-                      _["canopyBulkDensity"] = canopyBulkDensity,
-                      _["canopyLAI"] = canopyLAI);
-  Rcpp::DataFrame FM = DataFrame::create(_["fuelWind"]=fuelWind, 
-                                         _["fine1hFMC"]=fine1hFMC, 
-                                         _["coarse10hFMC"]=coarse10hFMC,
-                                         _["coarse100hFMC"]=coarse100hFMC,
-                                         _["liveHerbaceousFMC"] = fuelbedHerbaceousFMC,
-                                         _["liveWoodyFMC"] = fuelbedWoodyFMC,
-                                         _["canopyFMC"] = canopyFMC);
-
-  Rcpp::DataFrame FB = DataFrame::create(_["burningType"]=burningType,
-                                         _["midflameWind"] = midflameWind,
-                                         _["virtualWindSpeed"]=virtualWindSpeed,
-                                         _["spreadDirection"]=spreadDirection,
-                                         _["surfaceSpreadRate"]=surfaceSpreadRate,
-                                         _["intensityCrowningThreshold"]=I0,
-                                         _["spreadRate"]=spreadRate,
-                                         _["firelineIntensity"]=firelineIntensity);
-  
-  FS.attr("row.names") = meteo.attr("row.names");
-  FM.attr("row.names") = meteo.attr("row.names");
-  FB.attr("row.names") = meteo.attr("row.names");
-  List l = List::create(Named("FuelStructure")=FS, Named("FuelMoisture")=FM, Named("Behaviour")=FB);
-  l.attr("class") = CharacterVector::create("fb","list");
-  if(verbose) Rcout<<"done.\n";
-  return(l);  
-}
+// List fb(List x, List soil, double latitude, double slope, double aspect, DataFrame meteo, DataFrame SpParams, 
+//         DataFrame FuelModelParams, List control) {
+//   bool verbose = control["verbose"];
+//   String liveFMCmode = control["liveFMCmode"];
+//   bool useModelForLive = control["useModelForLive"]; 
+//   if(verbose) Rcout<<"Initializing";
+//   
+//   //Prepare swb input
+//   DataFrame swbInput = forest2swbInput(x, soil, SpParams, control);
+//   NumericVector pBole = swbInput["pBole"];
+//   NumericVector H = swbInput["H"];
+//   NumericVector Sgdd = swbInput["Sgdd"];
+//   IntegerVector SP = swbInput["SP"];
+//   NumericVector LAI = swbInput["LAI"];
+//   
+//   NumericVector cohLoading = cohortFuel(x, SpParams);
+// 
+//   //Initialize other cohort-based variables
+//   int numCohorts = LAI.size();
+//   NumericVector LAIphe(numCohorts), cohFMC(numCohorts), cohLoadingPhe(numCohorts);
+//   
+//   //Meteorological variables
+//   IntegerVector DOY = meteo["DOY"]; // day of the year
+//   NumericVector Precipitation = meteo["Precipitation"]; // in mm of water (L/m2)
+//   NumericVector MeanTemperature = meteo["MeanTemperature"]; // in degrees Celsius
+//   NumericVector MinTemperature = meteo["MinTemperature"]; // in degrees Celsius
+//   NumericVector MaxTemperature = meteo["MaxTemperature"]; // in degrees Celsius
+//   NumericVector RHmean = meteo["MeanRelativeHumidity"]; // in percentage
+//   NumericVector RHmin = meteo["MinRelativeHumidity"]; // in percentage
+//   NumericVector RHmax = meteo["MaxRelativeHumidity"]; // in percentage
+//   NumericVector WS = meteo["WindSpeed"]; // in m/s
+//   NumericVector WD = meteo["WindDirection"]; // in radians from north
+//   NumericVector Rn = meteo["Radiation"];  //in MJ/m2
+//   NumericVector PET = meteo["PET"]; //in mm of water
+// 
+//   NumericVector GDD = gdd(DOY, MeanTemperature, 5.0);
+//   NumericVector ER = er(DOY);
+// 
+//   int numDays = Precipitation.size();
+//   
+//   //Output FS variables
+//   NumericVector fuelbedHeight(numDays), fine1hLoading(numDays), 
+//                 coarse10hLoading(numDays), coarse100hLoading(numDays), 
+//                 herbaceousLoading(numDays), woodyLoading(numDays),
+//                 canopyBaseHeight(numDays), canopyTopHeight(numDays),
+//                 canopyLength(numDays), canopyBulkDensity(numDays),
+//                 canopyLAI(numDays);
+// 
+//   //Output FM variables
+//   NumericVector fuelWind(numDays), fine1hFMC(numDays), coarse10hFMC(numDays), 
+//                 coarse100hFMC(numDays), canopyFMC(numDays), fuelbedWoodyFMC(numDays),
+//                 fuelbedHerbaceousFMC(numDays);
+// 
+//   //Output FB variables
+//   IntegerVector burningType(numDays);
+//   NumericVector firelineIntensity(numDays), spreadRate(numDays), surfaceSpreadRate(numDays), 
+//                 midflameWind(numDays), virtualWindSpeed(numDays), spreadDirection(numDays), I0(numDays);
+//   
+//   //Initialize fuel moisture
+//   NumericVector fMoisture(5);
+//   fMoisture[0] = x["dead1hourMoisture"];
+//   fMoisture[1] = x["dead10hourMoisture"];
+//   fMoisture[2] = x["dead100hourMoisture"];
+//   
+//   NumericVector fcafternoonprev; //Stores fuel conditions of afternoon of previous day
+//   if(verbose) Rcout<<" - Daily loop ";
+//   for(int i=0;i<numDays;i++) {
+//     //SOIL WATER BALANCE
+//     if(verbose) Rcout<<".";
+//     List s = swbDay1(swbInput, soil, GDD[i], PET[i], Precipitation[i], ER[i], 0.0); //No Runon in simulations for a single cell    
+//     double Lground = s["Lground"]; //Percent light at the ground level
+// 
+//     //FUEL STRUCTURE
+//     NumericVector Phe = leafDevelopmentStatus(Sgdd, GDD[i]);
+//     for(int c=0;c<numCohorts;c++) {
+//       if(Sgdd[c]==0.0) Phe[c]=1.0;
+//       LAIphe[c] = LAI[c]*Phe[c]; //LAI modified by phenology 
+//       cohLoadingPhe[c] = cohLoading[c]*Phe[c]; //LAI modified by phenology 
+//     }
+//     //Loadings and FMC by cohort
+//     cohFMC = cohortFuelMoistureContent(s, swbInput, SpParams);
+//     
+//     List fs = fuelStructure(x, SpParams, FuelModelParams, GDD[i], 30, 5000, 0.01, useModelForLive);
+//     fuelbedHeight[i] = fs["fuelbedHeight [cm]"];
+//     NumericVector fBiomass = fs["fuelbedLoading [Mg/ha]"];
+//     NumericVector surfaceToVolumeRatios = fs["fuelbedSAV [m2/m3]"];
+//     fine1hLoading[i] = fBiomass[0];
+//     coarse10hLoading[i] = fBiomass[1];
+//     coarse100hLoading[i] = fBiomass[2];
+//     herbaceousLoading[i] = fBiomass[3];
+//     woodyLoading[i] = fBiomass[4];
+//     canopyBaseHeight[i] = fs["canopyBaseHeight [cm]"];
+//     canopyTopHeight[i] = fs["canopyTopHeight [cm]"];
+//     canopyLength[i] = fs["canopyLength [cm]"];
+//     canopyBulkDensity[i] = fs["canopyBulkDensity [kg/m3]"];
+//     canopyLAI[i] = fs["canopyLAI [dimensionless]"];
+// 
+//     //WIND EXTINCTION
+//     //Check NA
+//     if(NumericVector::is_na(WS[i])) WS[i] = 2.0; //Default value
+//     if(NumericVector::is_na(WD[i])) WD[i] = 180.0; //Wind from south
+//     
+//     //Calculates midflameWind
+//     if(NumericVector::is_na(canopyBaseHeight[i])) { //NO canopy over fire
+//        midflameWind[i] = unshelteredMidflameWindSpeed(WS[i], fuelbedHeight[i]/100.0); 
+//        fuelWind[i] = windSpeedAtCanopyHeight(WS[i], fuelbedHeight[i]/100.0);
+//     } else { //Wind extinction due to crowns
+//        midflameWind[i] = windSpeedMassmanExtinction(canopyBaseHeight[i]/100.0,WS[i], canopyLAI[i], canopyTopHeight[i]/100.0);
+//        fuelWind[i] = midflameWind[i];
+//     }    
+// 
+//     //FUEL MOISTURE
+//     double netPrec = s["NetPrec"];
+//     double rainDuration = std::min(24.0,Precipitation[i]/6.35);
+//     
+//     //Number of hours of insolation
+//     NumericVector srs = meteoland::radiation_sunRiseSet(latitude*(PI/180.0),  slope*(PI/180.0), aspect*(PI/180.0),DOY[i]);
+//     double n = std::max(0.0, (srs[1] - srs[0])*12.0/PI); 
+//     if(n<=0.0) n = 0.0;
+//     
+//     //Dead fuel moisture
+//     double radSec = Rn[i]*(Lground/100.0)*(1000000.0/(n*3600.0)); //Transform to MJ/m2.day to W/m2 assuming n hours of equal sunlight
+//     if(n<=0.0) radSec = 0.0;
+// 
+//     NumericVector fcmean = fuelConditions(MeanTemperature[i],RHmean[i], radSec, fuelWind[i]);
+//     NumericVector fcmorning = fuelConditions(MinTemperature[i],RHmax[i], radSec, fuelWind[i]);
+//     NumericVector fcafternoon = fuelConditions(MaxTemperature[i],RHmin[i], radSec, fuelWind[i]);
+//     if(i==0) fcafternoonprev = fcafternoon; 
+//     fine1hFMC[i] = fine1hday(fMoisture[0], fcmean[0], fcmean[1], fuelWind[i], netPrec);
+//     coarse10hFMC[i] = coarse10hday(fMoisture[1], 
+//                                    (fcafternoonprev[0] +fcmorning[0])/2.0, (fcafternoonprev[1] + fcmorning[1])/2.0,
+//                                    (fcmorning[0]+fcafternoon[0])/2.0, (fcmorning[1]+fcafternoon[1])/2.0, rainDuration);
+//     coarse100hFMC[i] = coarse100hday(fMoisture[2], 
+//                                      fcmorning[0], fcmorning[1],
+//                                      fcafternoon[0], fcafternoon[1], n, rainDuration);
+//     //Store for next day (and for fire behaviour)
+//     fMoisture[0] = fine1hFMC[i]; 
+//     fMoisture[1] =  coarse10hFMC[i]; 
+//     fMoisture[2] = coarse100hFMC[i]; 
+//     fcafternoonprev = fcafternoon;
+//       
+//     //Live fuel moisture
+//     NumericVector psi = s["psiVec"];
+//     if(liveFMCmode == "constant") {
+//       fMoisture[3] = 200.0; 
+//       fMoisture[4] = 100.0; 
+//       canopyFMC[i] = 100.0;
+//     } else if(liveFMCmode =="ffmc") {
+//       fMoisture[3] = fine1hFMC[i]; 
+//       fMoisture[4] = fine1hFMC[i]; 
+//       canopyFMC[i] = fine1hFMC[i];
+//     } else if(liveFMCmode =="swb") {
+//       //Level of physiological activity is defined in terms of growth degree days
+//       double act = leafDevelopmentStatus(200.0, GDD[i]);
+//       //In winter, herbaceous FMC is modelled as fine (1h) dead fuels
+//       //When physiologically active, FMC is modelled as function of water content in the topsoil
+//       fMoisture[3] = act*std::max(30.0, 200.0+20*(psi[0]/1000.0))+(1.0-act)*fine1hFMC[i]; 
+//       
+//       fMoisture[4] = fuelbedLiveFuelMoisture(fuelbedHeight[i], cohFMC, cohLoadingPhe, H, pBole); 
+//       canopyFMC[i] = canopyLiveFuelMoisture(canopyBaseHeight[i], canopyTopHeight[i], cohFMC, cohLoadingPhe, H, pBole); 
+//     } else {
+//       stop("Wrong 'liveFMCmode'");
+//     }
+//     
+//     fuelbedHerbaceousFMC[i] = fMoisture[3];
+//     fuelbedWoodyFMC[i] = fMoisture[4];
+// 
+// 
+//    CharacterVector FMcode = x["FuelModelCode"];
+//    CharacterVector models = FuelModelParams.attr("row.names");
+//    NumericVector heatContent(5);
+//    double deadFuelMoistureExtinction = 30.0; //Default value
+//    for(int j=0;j<models.length();j++) {
+//      if(models[j]==FMcode[0]) {
+//        heatContent[0] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_1h"])[j];
+//        heatContent[1] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_10h"])[j];
+//        heatContent[2] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_100h"])[j];
+//        heatContent[3] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_Live_Herb"])[j];
+//        heatContent[4] = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Heat_Live_Woody"])[j];
+//        deadFuelMoistureExtinction = Rcpp::as<Rcpp::NumericVector>(FuelModelParams["Mx_dead"])[j];
+//      }
+//    }
+//    
+//     //FIRE BEHAVIOUR
+//     //1. Calculate surface fire characteristics
+//     //1.2 Call to Rothermel model to obtain surface fire spread rate and intensity
+//     List rtm = rothermel("D", fBiomass, surfaceToVolumeRatios, fuelbedHeight[i],
+//                         deadFuelMoistureExtinction, heatContent, fMoisture, 
+//                         midflameWind[i], WD[i], 
+//                         slope, aspect);
+//     surfaceSpreadRate[i] = rtm["ROS [m/min]"];
+//     spreadRate[i] = surfaceSpreadRate[i];
+//     firelineIntensity[i] = rtm["Fireline intensity [kW/m]"];
+//     virtualWindSpeed[i] = rtm["Virtual wind speed [m/s]"];
+//     NumericVector slopeWindVec = rtm["Slope-wind vector"];
+//     
+//     //1.3 Determine whether crowning will occur  (Van Wagner 1989)
+//     if(!NumericVector::is_na(canopyBaseHeight[i])) {
+//       I0[i] = criticalFirelineIntensity(canopyBaseHeight[i]/100.0, canopyFMC[i]); //critical fireline intensity (MW/m)
+//       //Is crown fire (active or passive)?
+//       if(firelineIntensity[i] >= I0[i]) { 
+//         double RAC = 3.0/canopyBulkDensity[i]; // Threshold for active crown spread rate (m/min)
+//         double Ei = 1.0;
+//         NumericVector fBiomass10 = NumericVector::create(0.38, 0.092, 0.230,0.0, 0.092); //lb/square feet
+//         fBiomass10 = fBiomass10*(1.0/(0.3048*0.3048))*0.45359237*(10.0); // transform to tons/hectare
+//         List rtm10 = rothermel("D", fBiomass10, surfaceToVolumeRatios, 30.48,
+//                         25, heatContent, fMoisture, WS[i]*0.4, WD[i], 
+//                         slope, aspect);
+//         double R10 = rtm10["ROS [m/min]"]; //Spread rate with fuel model 10 (Rothermel 1991)
+//         double R0 = (I0[i]/firelineIntensity[i])*surfaceSpreadRate[i]; //critical surface spread rate (m/s) associated with critical fireline intensity
+//         double ac = -log(0.1)/(0.9*(RAC-R0)); //Scaling coefficient
+//         double cFractionBurn = 1.0 - exp(-ac*(surfaceSpreadRate[i]-R0)); //Fraction of canopy burn with crowning
+//         double Rcmax = (3.34)*R10*Ei; // Maximum crown spread rate (m/s)
+//         double Rcactual = surfaceSpreadRate[i] + cFractionBurn*(Rcmax - surfaceSpreadRate[i]); //Actual crown spread rate (m/s)
+//         if(Rcactual >= RAC) {
+//           burningType[i] = 4;
+//           spreadRate[i] = Rcactual;
+//         } else {
+//           burningType[i] = 3;
+//         }
+//         //Modify fireline intensity for crown fire
+//         firelineIntensity[i] = 300.0 * ((firelineIntensity[i]/(300.0*surfaceSpreadRate[i])) + cFractionBurn*canopyBulkDensity[i]*(canopyTopHeight[i]-canopyBaseHeight[i])*0.01)*spreadRate[i];        
+//       } else {
+//         burningType[i] = 2; // Surface fire
+//       }
+//     } else {
+//       burningType[i] = 0; // Shrubland (open) fire
+//     }
+// 
+//     spreadDirection[i] = slopeWindVec[1]*(180.0/3.141592);  
+//     if(spreadDirection[i]<0) spreadDirection[i] +=360.0; 
+//   }
+//   
+//   if(verbose) Rcout<<"Building output ...";  
+//   Rcpp::DataFrame FS = DataFrame::create(_["fuelbedHeight"] = fuelbedHeight,
+//                       _["fine1hLoading"] = fine1hLoading,
+//                       _["coarse10hLoading"] = coarse10hLoading,
+//                       _["coarse100hLoading"] = coarse100hLoading,
+//                       _["herbaceousLoading"] = herbaceousLoading,
+//                       _["woodyLoading"] = woodyLoading,
+//                       _["canopyBaseHeight"] = canopyBaseHeight,
+//                       _["canopyTopHeight"] = canopyTopHeight,
+//                       _["canopyLength"] = canopyLength,
+//                       _["canopyBulkDensity"] = canopyBulkDensity,
+//                       _["canopyLAI"] = canopyLAI);
+//   Rcpp::DataFrame FM = DataFrame::create(_["fuelWind"]=fuelWind, 
+//                                          _["fine1hFMC"]=fine1hFMC, 
+//                                          _["coarse10hFMC"]=coarse10hFMC,
+//                                          _["coarse100hFMC"]=coarse100hFMC,
+//                                          _["liveHerbaceousFMC"] = fuelbedHerbaceousFMC,
+//                                          _["liveWoodyFMC"] = fuelbedWoodyFMC,
+//                                          _["canopyFMC"] = canopyFMC);
+// 
+//   Rcpp::DataFrame FB = DataFrame::create(_["burningType"]=burningType,
+//                                          _["midflameWind"] = midflameWind,
+//                                          _["virtualWindSpeed"]=virtualWindSpeed,
+//                                          _["spreadDirection"]=spreadDirection,
+//                                          _["surfaceSpreadRate"]=surfaceSpreadRate,
+//                                          _["intensityCrowningThreshold"]=I0,
+//                                          _["spreadRate"]=spreadRate,
+//                                          _["firelineIntensity"]=firelineIntensity);
+//   
+//   FS.attr("row.names") = meteo.attr("row.names");
+//   FM.attr("row.names") = meteo.attr("row.names");
+//   FB.attr("row.names") = meteo.attr("row.names");
+//   List l = List::create(Named("FuelStructure")=FS, Named("FuelMoisture")=FM, Named("Behaviour")=FB);
+//   l.attr("class") = CharacterVector::create("fb","list");
+//   if(verbose) Rcout<<"done.\n";
+//   return(l);  
+// }
