@@ -1,6 +1,6 @@
 #include <numeric>
 #include <Rcpp.h>
-#include "swb.h"
+#include "spwb.h"
 #include "forestutils.h"
 #include "fuelmoisture.h"
 using namespace Rcpp;
@@ -47,8 +47,7 @@ const double broadleavedRPR = 10.31; //unitless
  * Returns the proportion of the crown (Hbc - H) that lies within given interval (zLow-zHigh)
  */
 double crownProportionInLayer(double zLow, double zHigh, double H, double Hbc) {
-  if((H-Hbc)>0) return((std::max(0.0, std::min(H, zHigh) - std::max(Hbc, zLow)))/(H-Hbc));
-  return(0.0);
+  return(leafAreaProportion(zLow, zHigh, Hbc, H));
 }
 /**
  * Returns the fuel loading (kg/m2) of the crown (Hbc - H) that lies within given interval (zLow-zHigh)
@@ -119,7 +118,7 @@ double layerLAI(double minHeight, double maxHeight, NumericVector cohortLAI, Num
 // [[Rcpp::export(".layerFuelAverageSpeciesParameter")]]
 double layerFuelAverageSpeciesParameter(String spParName, double minHeight, double maxHeight, List x, DataFrame SpParams, double gdd = NA_REAL) {
   NumericVector cohortLoading = cohortFuel(x, SpParams, gdd); //in kg/m2
-  NumericVector parValues = cohortParameter(x, SpParams, spParName);
+  NumericVector parValues = cohortNumericParameter(x, SpParams, spParName);
   NumericVector CR = cohortCrownRatio(x, SpParams);
   NumericVector H = cohortHeight(x);
   double num = 0.0, den = 0.0, cfl = 0.0;
@@ -132,6 +131,7 @@ double layerFuelAverageSpeciesParameter(String spParName, double minHeight, doub
   if(den>0) return(num/den);
   return(NA_REAL);
 }
+
 // [[Rcpp::export(".layerFuelAverageParameter")]]
 double layerFuelAverageParameter(double minHeight, double maxHeight, NumericVector cohortParameter, NumericVector cohortLoading, NumericVector H, NumericVector CR) {
   double num = 0.0, den = 0.0, cfl = 0.0;
@@ -213,7 +213,7 @@ List fuelLiveStratification(List object, DataFrame SpParams, double gdd = NA_REA
  * FCCS fuel definition
  */
 // [[Rcpp::export("fuel.FCCS")]]
-DataFrame FCCSproperties(List object, double ShrubCover, double CanopyCover, DataFrame SpParams, double gdd = NA_REAL, 
+DataFrame FCCSproperties(List object, double ShrubCover, double CanopyCover, DataFrame SpParams, NumericVector cohortFMC = NumericVector::create(), double gdd = NA_REAL, 
                    double heightProfileStep = 10.0, double maxHeightProfile = 5000, double bulkDensityThreshold = 0.05) {
   List liveStrat = fuelLiveStratification(object, SpParams, gdd, heightProfileStep, maxHeightProfile, bulkDensityThreshold);
   
@@ -223,14 +223,14 @@ DataFrame FCCSproperties(List object, double ShrubCover, double CanopyCover, Dat
   NumericVector cohHeight = cohortHeight(object);
   NumericVector cohCL = cohortCrownLength(object, SpParams);
     
-  NumericVector cohParticleDensity = cohortParameter(object, SpParams, "ParticleDensity");
-  NumericVector cohSAV = cohortParameter(object, SpParams, "SAV");
-  NumericVector cohFlammability = cohortParameter(object, SpParams, "Flammability");
-  NumericVector cohHeatContent = cohortParameter(object, SpParams, "HeatContent");
+  NumericVector cohParticleDensity = cohortNumericParameter(object, SpParams, "ParticleDensity");
+  NumericVector cohSAV = cohortNumericParameter(object, SpParams, "SAV");
+  NumericVector cohFlammability = cohortNumericParameter(object, SpParams, "Flammability");
+  NumericVector cohHeatContent = cohortNumericParameter(object, SpParams, "HeatContent");
   NumericVector cohCR = cohortCrownRatio(object, SpParams);
-  NumericVector cohMinFMC = cohortParameter(object, SpParams, "minFMC");
-  NumericVector cohMaxFMC = cohortParameter(object, SpParams, "maxFMC");
-  NumericVector cohpDead = cohortParameter(object, SpParams, "pDead");
+  NumericVector cohMinFMC = cohortNumericParameter(object, SpParams, "minFMC");
+  NumericVector cohMaxFMC = cohortNumericParameter(object, SpParams, "maxFMC");
+  NumericVector cohpDead = cohortNumericParameter(object, SpParams, "pDead");
   CharacterVector leafLitterType = cohortCharacterParameter(object, SpParams, "LeafLitterFuelType");
 
   //Canopy limits and loading  
@@ -338,6 +338,14 @@ DataFrame FCCSproperties(List object, double ShrubCover, double CanopyCover, Dat
   else maxFMC[0] = NA_REAL;
   if(shrubLoading>0.0) maxFMC[1] = layerFuelAverageParameter(0.0, 200.0, cohMaxFMC, cohLoading, cohHeight, cohCR);
   else maxFMC[1] = NA_REAL;
+  
+  NumericVector actFMC(5,NA_REAL); //Actual FMC
+  if(cohortFMC.size()==cohLoading.size()) {
+    if(canopyLoading>0.0) actFMC[0] = layerFuelAverageParameter(200.0, 10000.0, cohortFMC, cohLoading, cohHeight, cohCR);
+    else actFMC[0] = NA_REAL;
+    if(shrubLoading>0.0) actFMC[1] = layerFuelAverageParameter(0.0, 200.0, cohortFMC, cohLoading, cohHeight, cohCR);
+    else actFMC[1] = NA_REAL;
+  }
   NumericVector RV(5,0.0); //Reactive volume (m3/m2)
   double wmaxli = 0.0;
   double rhoblitter = 0.0, SAVlitter = 0.0, wmaxlitter = 0.0, betarellitter = 0.0, etalitter = 0.0;
@@ -457,7 +465,8 @@ DataFrame FCCSproperties(List object, double ShrubCover, double CanopyCover, Dat
                                                      _["etaF"] = etaF,
                                                      _["RV"] = RV,
                                                      _["MinFMC"] = minFMC,
-                                                     _["MaxFMC"] = maxFMC);
+                                                     _["MaxFMC"] = maxFMC,
+                                                     _["ActFMC"] = actFMC);
   FCCSProperties.attr("row.names") = CharacterVector::create("canopy","shrub", "herb", "woody","litter");
   return(FCCSProperties);
 }
@@ -582,8 +591,8 @@ List fuelStructure(List object, DataFrame SpParams, DataFrame FuelModelParams, d
   }
   
   NumericVector cohortLoading = cohortFuel(object, SpParams, gdd);
-  NumericVector minFMC = cohortParameter(object, SpParams, "minFMC");
-  NumericVector maxFMC = cohortParameter(object, SpParams, "maxFMC");
+  NumericVector minFMC = cohortNumericParameter(object, SpParams, "minFMC");
+  NumericVector maxFMC = cohortNumericParameter(object, SpParams, "maxFMC");
   NumericVector CR = cohortCrownRatio(object, SpParams);
   NumericVector H = cohortHeight(object);
   
@@ -598,8 +607,8 @@ List fuelStructure(List object, DataFrame SpParams, DataFrame FuelModelParams, d
     }
     fbLoading[4] = fs["fuelBedWoodyBiomass"];
     fbHeight = fs["fuelbedHeight"];
-    NumericVector cohortSAV = cohortParameter(object, SpParams, "SAV");
-    NumericVector cohortHeatContent = cohortParameter(object, SpParams, "HeatContent");
+    NumericVector cohortSAV = cohortNumericParameter(object, SpParams, "SAV");
+    NumericVector cohortHeatContent = cohortNumericParameter(object, SpParams, "HeatContent");
     //Herbaceous fuel
     fbSurfaceToVolumeRatio[3] = 4921.0;
     fbHeatContent[3] = 18622;
